@@ -14,28 +14,24 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
+#include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
-#include <sys/types.h>
 #include <dirent.h>
-#include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <libgen.h>
 #include <limits.h>
-#include <uuid/uuid.h>
 #include <ctype.h>
-
+#include <uuid/uuid.h>
 #include <gd.h>
-
-#undef ULONG_MAX
-
-#include "kerncompat.h"
-#include "ctree.h"
-#include "ioctl.h"
+#include "kernel-shared/ctree.h"
 #include "common/utils.h"
+#include "ioctl.h"
 
 static int use_color;
 static void
