@@ -16,15 +16,18 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <sys/ioctl.h>
 #include <limits.h>
 #include <errno.h>
-
-#include "kernel-shared/ctree.h"
+#include "kernel-lib/rbtree.h"
+#include "libbtrfs/ctree.h"
 #include "libbtrfs/send-utils.h"
-#include "ioctl.h"
+#include "libbtrfs/ioctl.h"
 
 static int btrfs_subvolid_resolve_sub(int fd, char *path, size_t *path_len,
 				      u64 subvol_id);
@@ -469,6 +472,10 @@ void subvol_uuid_search_add(struct subvol_uuid_search *s,
 }
 #endif
 
+static struct subvol_info *subvol_uuid_search2(struct subvol_uuid_search *s,
+				       u64 root_id, const u8 *uuid, u64 transid,
+				       const char *path,
+				       enum subvol_search_type type);
 struct subvol_info *subvol_uuid_search(struct subvol_uuid_search *s,
 				       u64 root_id, const u8 *uuid, u64 transid,
 				       const char *path,
@@ -553,7 +560,7 @@ out:
 	return ret;
 }
 
-struct subvol_info *subvol_uuid_search2(struct subvol_uuid_search *s,
+static struct subvol_info *subvol_uuid_search2(struct subvol_uuid_search *s,
 				       u64 root_id, const u8 *uuid, u64 transid,
 				       const char *path,
 				       enum subvol_search_type type)
