@@ -9,7 +9,9 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public
- * License along with this program.
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 021110-1307, USA.
  */
 
 /*
@@ -19,11 +21,17 @@
 #ifndef __BTRFS_MKFS_ROOTDIR_H__
 #define __BTRFS_MKFS_ROOTDIR_H__
 
+#include "kerncompat.h"
+#include <sys/types.h>
+#include <stdbool.h>
 #include "kernel-lib/list.h"
 
+struct btrfs_fs_info;
+struct btrfs_root;
+
 struct directory_name_entry {
 	const char *dir_name;
-	const char *path;
+	char *path;
 	ino_t inum;
 	struct list_head list;
 };
